/** 
 ** @file   VGA.c
 ** @date   2012-06-23
 ** @author Johannes Heimansberg (wejp.k.vu)
 **
 ** @brief  A simple Pong game for the STM32F4-Discovery board
 **         with external VGA DAC and accelerometer control.
 **         This software VGA implementation is based on the 
 **         implementation used in the Peridiummmm demo and has been
 **         ported by me to libopencm3.
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; version 2 of
 ** the License. See the file COPYING for details.
 **/
#include "VGA.h"
#include "libopencm3/stm32/f4/gpio.h"
#include "libopencm3/stm32/f4/rcc.h"
#include "libopencm3/stm32/f4/dma.h"
#include "libopencm3/stm32/nvic.h"
#include <stdlib.h>

static void InitializeState();
static void BlankHSyncHandler480();
static void BlankHSyncHandler400();
static void BlankHSyncHandler350();
static void PixelHSyncHandler240();
static void PixelHSyncHandler200();
static void PixelHSyncHandler175();
static void PixelHSyncHandler160();
static void PixelHSyncHandler133();
static void PixelHSyncHandler117();

static void InitializePixelDMA(int pixelclock,int pixelsperrow);
static void DMACompleteHandler();
static inline void StartPixelDMA();
static inline void StopPixelDMA();

uint32_t VGALine;
volatile uint32_t VGAFrame;
uint32_t VGAFrameBufferAddress;
uint32_t VGACurrentLineAddress;
uint32_t VGAPixelsPerRow;
uint16_t VGAThreeLineCounter;

static HBlankInterruptFunction *HBlankInterruptHandler;

void InitializeVGAScreenMode480()
{
	HBlankInterruptHandler=NULL;
	InitializeVGAPort();
	InitializeState();
	SetBlankVGAScreenMode480();
}

void InitializeVGAScreenMode400()
{
	HBlankInterruptHandler=NULL;
	InitializeVGAPort();
	InitializeState();
	SetBlankVGAScreenMode400();
}

void InitializeVGAScreenMode350()
{
	HBlankInterruptHandler=NULL;
	InitializeVGAPort();
	InitializeState();
	SetBlankVGAScreenMode350();
}

void SetBlankVGAScreenMode480()
{
	SetVGASignalToBlack();
	InitializeVGAHorizontalSync31kHz(BlankHSyncHandler480);
}

void SetBlankVGAScreenMode400()
{
	SetVGASignalToBlack();
	InitializeVGAHorizontalSync31kHz(BlankHSyncHandler400);
}

void SetBlankVGAScreenMode350()
{
	SetVGASignalToBlack();
	InitializeVGAHorizontalSync31kHz(BlankHSyncHandler350);
}

void SetVGAScreenMode240(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	SetVGAHorizontalSync31kHz(PixelHSyncHandler240);
}

void SetVGAScreenMode200(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	SetVGAHorizontalSync31kHz(PixelHSyncHandler200);
}

void SetVGAScreenMode175(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	SetVGAHorizontalSync31kHz(PixelHSyncHandler175);
}

void SetVGAScreenMode160(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	VGAThreeLineCounter=0;
	SetVGAHorizontalSync31kHz(PixelHSyncHandler160);
}

void SetVGAScreenMode133(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	VGAThreeLineCounter=0;
	SetVGAHorizontalSync31kHz(PixelHSyncHandler133);
}

void SetVGAScreenMode117(uint8_t *framebuffer,int pixelsperrow,int pixelclock)
{
	WaitVBL();
	InitializePixelDMA(pixelclock,pixelsperrow);
	SetFrameBuffer(framebuffer);
	VGAThreeLineCounter=0;
	SetVGAHorizontalSync31kHz(PixelHSyncHandler117);
}

void InitializeVGAPort()
{
	// Turn on peripherals.
	rcc_peripheral_enable_clock(&RCC_AHB1ENR, RCC_AHB1ENR_IOPBEN|RCC_AHB1ENR_IOPEEN|RCC_AHB1ENR_DMA2EN);
	rcc_peripheral_enable_clock(&RCC_APB2ENR, RCC_APB2ENR_TIM8EN|RCC_APB2ENR_SYSCFGEN);
	rcc_peripheral_enable_clock(&RCC_APB1ENR, RCC_APB1ENR_TIM3EN);

	// Configure DAC pins, and set to black.
	gpio_set_output_options(GPIOE, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO8|GPIO9|GPIO10|GPIO11|GPIO12|GPIO13|GPIO14|GPIO15);
	gpio_mode_setup(GPIOE, GPIO_MODE_OUTPUT, GPIO_PUPD_PULLUP, GPIO8|GPIO9|GPIO10|GPIO11|GPIO12|GPIO13|GPIO14|GPIO15);
	gpio_clear(GPIOE, GPIO8|GPIO9|GPIO10|GPIO11|GPIO12|GPIO13|GPIO14|GPIO15);

	// Configure sync pins and drive them high.
	gpio_mode_setup(GPIOB, GPIO_MODE_OUTPUT, GPIO_PUPD_PULLUP, GPIO11|GPIO12);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO11|GPIO12);
	gpio_set(GPIOB, GPIO11|GPIO12);
}

void InitializeVGAHorizontalSync31kHz(InterruptHandler *handler)
{
	// Configure timer 3 (was 9) as the HSync timer.
	timer_reset(TIM3);
	TIM3_CR1   = TIM_CR1_ARPE|TIM_CR1_DIR_UP;
	TIM3_DIER  = TIM_DIER_UIE|TIM_DIER_CC1IE|TIM_DIER_CC2IE; // Enable update, compare 1 and 2 interrupts.
	TIM3_CCMR1 = 0;
	TIM3_CCER  = 0;
	TIM3_PSC   = 0;    // Prescaler = 1
	TIM3_ARR   = 2669; // 84 MHz / 31.4686 kHz = 2669.32752
	TIM3_CCR1  = 317;  // 84 MHz * 3.77 microseconds = 316.68 - sync pulse end
	TIM3_CCR2  = 475;  // 84 MHz * (3.77 + 1.89) microseconds = 475.44 - back porch end

	SetVGAHorizontalSync31kHz(handler);

	// Enable HSync timer.
	TIM3_CR1 |= TIM_CR1_CEN;
}

void SetVGAHorizontalSync31kHz(InterruptHandler *handler)
{
	/* Enable HSync timer interrupt and set highest priority. */
	InstallInterruptHandler(NVIC_TIM3_IRQ, handler);
	EnableInterrupt(NVIC_TIM3_IRQ);
	SetInterruptPriority(NVIC_TIM3_IRQ, 0);	
}

void SetHBlankInterruptHandler(HBlankInterruptFunction *handler)
{
	HBlankInterruptHandler=handler;
}

static void InitializeState() //uint8_t *framebuffer,int pixelsperrow)
{
	VGALine  = 0xffffffff;
	VGAFrame = 0;
}

static void BlankHSyncHandler480()
{
	HandleVGAHSync480();
}

static void BlankHSyncHandler400()
{
	HandleVGAHSync400();
}

static void BlankHSyncHandler350()
{
	HandleVGAHSync350();
}

static void PixelHSyncHandler240()
{
	int line=HandleVGAHSync480();
	if(line<0) return;

	StartPixelDMA();
	if(line&1) VGACurrentLineAddress+=VGAPixelsPerRow;
}

static void PixelHSyncHandler200()
{
	int line=HandleVGAHSync400();
	if(line<0) return;

	StartPixelDMA();
	if(line&1) VGACurrentLineAddress+=VGAPixelsPerRow;
}

static void PixelHSyncHandler175()
{
	int line=HandleVGAHSync350();
	if(line<0) return;

	StartPixelDMA();
	if(line&1) VGACurrentLineAddress+=VGAPixelsPerRow;
}

static void PixelHSyncHandler160()
{
	int line=HandleVGAHSync480();
	if(line<0) return;

	StartPixelDMA();
	if(VGAThreeLineCounter++==2)
	{
		VGACurrentLineAddress+=VGAPixelsPerRow;
		VGAThreeLineCounter=0;
	}
}

static void PixelHSyncHandler133()
{
	int line=HandleVGAHSync400();
	if(line<0) return;

	StartPixelDMA();
	if(VGAThreeLineCounter++==2)
	{
		VGACurrentLineAddress+=VGAPixelsPerRow;
		VGAThreeLineCounter=0;
	}
}

static void PixelHSyncHandler117()
{
	int line=HandleVGAHSync350();
	if(line<0) return;

	StartPixelDMA();
	if(VGAThreeLineCounter++==2)
	{
		VGACurrentLineAddress+=VGAPixelsPerRow;
		VGAThreeLineCounter=0;
	}
}

static void InitializePixelDMA(int pixelclock,int pixelsperrow)
{
	// Configure timer 8 as the pixel clock.
	timer_reset(TIM8);
	TIM8_CR1  = TIM_CR1_ARPE;
	TIM8_DIER = TIM_DIER_UDE; // Enable update DMA request.
	TIM8_PSC  = 0;            // Prescaler = 1
	TIM8_ARR  = pixelclock-1; // TODO: Should this be -1?

	// DMA2 stream 1 channel 7 is triggered by timer 8.
	// Stop it and configure interrupts.
	DMA2_S1CR &= ~DMA_SxCR_EN;
	InstallInterruptHandler(NVIC_DMA2_STREAM1_IRQ, DMACompleteHandler);
	nvic_enable_irq(NVIC_DMA2_STREAM1_IRQ);
	nvic_set_priority(NVIC_DMA2_STREAM1_IRQ, 0);

	VGAPixelsPerRow = pixelsperrow;
}

static void DMACompleteHandler()
{
	SetVGASignalToBlack();
	DMA2_LIFCR |= DMA_LIFCR_CTCIF1; /* Clear interrupt flag. */
	StopPixelDMA();

	if (HBlankInterruptHandler) HBlankInterruptHandler();
	// TODO: VBlank interrupt? At lower priority?
	static int i = 10000;
	if (!i--) { i = 10000; gpio_toggle(GPIOD, GPIO14); } // red LED
}

static inline void StartPixelDMA()
{
	/* Visible line. Configure and enable pixel DMA. */
	DMA2_S1CR = DMA_SxCR_CHSEL(7)|              // Channel 7
	            DMA_SxCR_PL_LOW|                // Priority 3
	            DMA_SxCR_PSIZE_8BIT|            // PSIZE = 8 bit
	            DMA_SxCR_MSIZE_8BIT|            // MSIZE = 8 bit
	            DMA_SxCR_MINC|                  // Increase memory address
	            DMA_SxCR_DIR_MEM_TO_PERIPHERAL| // Memory to peripheral
	            DMA_SxCR_TCIE;                  // Transfer complete interrupt

	DMA2_S1NDTR = VGAPixelsPerRow;
	DMA2_S1PAR  = (volatile uint32_t*)(GPIOE+0x14+1);
	DMA2_S1M0AR = (volatile uint32_t*)VGACurrentLineAddress;
	DMA2_S1CR   |= DMA_SxCR_EN;

	/* Start pixel clock. */
    TIM8_CNT = 0;
	TIM8_CR1 |= TIM_CR1_CEN;
}

static inline void StopPixelDMA()
{
	TIM8_CR1 &= ~TIM_CR1_CEN; /* Stop pixel clock.  */
	DMA2_S1CR = 0;            /* Disable pixel DMA. */
}
