/** 
 ** @file   VGA.h
 ** @date   2012-06-23
 ** @author Johannes Heimansberg (wejp.k.vu)
 **
 ** @brief  A simple Pong game for the STM32F4-Discovery board
 **         with external VGA DAC and accelerometer control.
 **         This software VGA implementation is based on the 
 **         implementation used in the Peridiummmm demo and has been
 **         ported by me to libopencm3.
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; version 2 of
 ** the License. See the file COPYING for details.
 **/
#ifndef WEJ_VGA_H
#define WEJ_VGA_H
#include "System.h"
#include <libopencm3/stm32/f4/timer.h>
#include <libopencm3/stm32/f4/gpio.h>
#include <stdint.h>

typedef void HBlankInterruptFunction(void);

extern uint32_t VGALine;
extern volatile uint32_t VGAFrame;
extern uint32_t VGAFrameBufferAddress;
extern uint32_t VGACurrentLineAddress;
extern uint32_t VGAPixelsPerRow;

void InitializeVGAScreenMode480();
void InitializeVGAScreenMode400();
void InitializeVGAScreenMode350();

void SetBlankVGAScreenMode480();
void SetBlankVGAScreenMode400();
void SetBlankVGAScreenMode350();

void SetVGAScreenMode240(uint8_t *framebuffer,int pixelsperrow,int pixelclock);
void SetVGAScreenMode200(uint8_t *framebuffer,int pixelsperrow,int pixelclock);
void SetVGAScreenMode175(uint8_t *framebuffer,int pixelsperrow,int pixelclock);

void SetVGAScreenMode160(uint8_t *framebuffer,int pixelsperrow,int pixelclock);
void SetVGAScreenMode133(uint8_t *framebuffer,int pixelsperrow,int pixelclock);
void SetVGAScreenMode117(uint8_t *framebuffer,int pixelsperrow,int pixelclock);

static inline void SetVGAScreenMode320x240(uint8_t *framebuffer) { SetVGAScreenMode240(framebuffer,320,13); }
static inline void SetVGAScreenMode320x200(uint8_t *framebuffer) { SetVGAScreenMode200(framebuffer,320,13); }
static inline void SetVGAScreenMode320x175(uint8_t *framebuffer) { SetVGAScreenMode175(framebuffer,320,13); }

static inline void SetVGAScreenMode212x160(uint8_t *framebuffer) { SetVGAScreenMode160(framebuffer,212,19); }
static inline void SetVGAScreenMode212x133(uint8_t *framebuffer) { SetVGAScreenMode133(framebuffer,212,19); }
static inline void SetVGAScreenMode212x175(uint8_t *framebuffer) { SetVGAScreenMode117(framebuffer,212,19); }

static inline void WaitVBL()
{
	uint32_t currframe=VGAFrame;
	while(VGAFrame==currframe);
}

static inline void SetFrameBuffer(uint8_t *framebuffer)
{
	VGAFrameBufferAddress=VGACurrentLineAddress=(uint32_t)framebuffer;
}

static inline uint32_t VGAFrameCounter() { return VGAFrame; }


// Functions for implementing your own video driving code.

void InitializeVGAPort();
void InitializeVGAHorizontalSync31kHz(InterruptHandler *handler);
void SetVGAHorizontalSync31kHz(InterruptHandler *handler);

#define VGAHorizontalSyncStartInterruptFlag TIM_DIER_UIE   // Overflow interrupt
#define VGAHorizontalSyncEndInterruptFlag   TIM_DIER_CC1IE // Output compare 1 interrupt
#define VGAVideoStartInterruptFlag          TIM_DIER_CC2IE // Output compare 2 interrupt

static inline void RaiseVGAHSyncLine()
{
	gpio_set(GPIOB, GPIO11);
}

static inline void LowerVGAHSyncLine()
{
	gpio_clear(GPIOB, GPIO11);
}

static inline void RaiseVGAVSyncLine()
{
	gpio_set(GPIOB, GPIO12);
}

static inline void LowerVGAVSyncLine()
{
	gpio_clear(GPIOB, GPIO12);
}

static inline void SetVGASignalToBlack()
{
	gpio_clear(GPIOE, GPIO8|GPIO9|GPIO10|GPIO11|GPIO12|GPIO13|GPIO14|GPIO15);
}

static inline void SetVGASignal(uint8_t pixel)
{
	//((uint8_t *)&GPIOE->ODR)[1]=pixel;
	((uint8_t *)&GPIOE_ODR)[1] = pixel;
}

static inline int HandleVGAHSync480()
{
	uint32_t sr = TIM3_SR;
	TIM3_SR = 0;

	if (sr & VGAHorizontalSyncStartInterruptFlag)    LowerVGAHSyncLine();
	else if (sr & VGAHorizontalSyncEndInterruptFlag) RaiseVGAHSyncLine();
	else // if(VGAVideoStartInterruptFlag)
	{
		VGALine++;
		if(VGALine<480)
		{
			return VGALine;
		}
		else if(VGALine==480)
		{
			VGAFrame++;
		}
		else if(VGALine==490)
		{
			LowerVGAVSyncLine();
		}
		else if(VGALine==492)
		{
			RaiseVGAVSyncLine();
		}
		else if(VGALine==524)
		{
			VGALine=0;
			VGACurrentLineAddress=VGAFrameBufferAddress;
		}
	}
	return -1;
}

static inline int HandleVGAHSync400()
{
	uint32_t sr = TIM3_SR;
	TIM3_SR = 0;

	if (sr & VGAHorizontalSyncStartInterruptFlag)    LowerVGAHSyncLine();
	else if (sr & VGAHorizontalSyncEndInterruptFlag) RaiseVGAHSyncLine();
	else // if(VGAVideoStartInterruptFlag)
	{
		VGALine++;
		if(VGALine<400)
		{
			return VGALine;
		}
		else if(VGALine==400)
		{
			VGAFrame++;
		}
		else if(VGALine==412)
		{
			RaiseVGAVSyncLine();
		}
		else if(VGALine==414)
		{
			LowerVGAVSyncLine();
		}
		else if(VGALine==448)
		{
			VGALine=0;
			VGACurrentLineAddress=VGAFrameBufferAddress;
		}
	}
	return -1;
}

static inline int HandleVGAHSync350()
{
	uint32_t sr = TIM3_SR;
	TIM3_SR = 0;

	if (sr & VGAHorizontalSyncStartInterruptFlag)    RaiseVGAHSyncLine();
	else if (sr & VGAHorizontalSyncEndInterruptFlag) LowerVGAHSyncLine();
	else // if(VGAVideoStartInterruptFlag)
	{
		VGALine++;
		if(VGALine<350)
		{
			return VGALine;
		}
		else if(VGALine==350)
		{
			VGAFrame++;
		}
		else if(VGALine==387)
		{
			LowerVGAVSyncLine();
		}
		else if(VGALine==389)
		{
			RaiseVGAVSyncLine();
		}
		else if(VGALine==448)
		{
			VGALine=0;
			VGACurrentLineAddress=VGAFrameBufferAddress;
		}
	}
	return -1;
}

#endif
