/** 
 ** @file   accelerometer.c
 ** @date   2012-06-23
 ** @author Johannes Heimansberg (wejp.k.vu)
 **
 ** @brief  A simple Pong game for the STM32F4-Discovery board
 **         with external VGA DAC and accelerometer control.
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; version 2 of
 ** the License. See the file COPYING for details.
 **/
#include "accelerometer.h"
#include "spi.h"
#include "libopencm3/stm32/f4/gpio.h"
#include "libopencm3/stm32/f4/rcc.h"
#include "libopencm3/stm32/f4/spi.h"

void accelerometer_init()
{
	/* Enable peripherals. */
	rcc_peripheral_enable_clock(&RCC_AHB1ENR, RCC_AHB1ENR_IOPAEN|RCC_AHB1ENR_IOPEEN);
	rcc_peripheral_enable_clock(&RCC_APB2ENR, RCC_APB2ENR_SPI1EN);

	/* Configure SPI pins as alternate function. */
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO5|GPIO6|GPIO7);
	gpio_set_af(GPIOA, GPIO_AF5, GPIO5|GPIO6|GPIO7);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO5|GPIO6|GPIO7);

	/* Configure and enable SPI. */
	rcc_peripheral_reset(&RCC_APB2RSTR, RCC_APB2RSTR_SPI1RST);
	rcc_peripheral_clear_reset(&RCC_APB2RSTR, RCC_APB2RSTR_SPI1RST);

	SPI1_I2SCFGR &= ~SPI_I2SCFGR_I2SMOD; /* Disable I2S mode. */
	SPI1_CR1 = SPI_CR1_SSM|SPI_CR1_BAUDRATE_FPCLK_DIV_4|SPI_CR1_SSI|SPI_CR1_MSTR|SPI_CR1_SPE;

	/* Configure CS pin as input to be compatible with VGA driver.
	 * Set the output register to 0 anyway, and reconfigure later in lower_cs(). */
	gpio_set_output_options(GPIOE, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO3);
	gpio_mode_setup(GPIOE, GPIO_MODE_INPUT, GPIO_PUPD_PULLUP, GPIO3);
	gpio_clear(GPIOE, GPIO3);

	/* Configure interrupt pins. */
	gpio_mode_setup(GPIOE, GPIO_MODE_INPUT, GPIO_PUPD_NONE, GPIO0|GPIO1);
}

bool accelerometer_ping()
{
	uint8_t byte = wspi_read_byte(LIS302DL_WHO_AM_I_ADDR, GPIOE, GPIO3);
	return byte == 0x3b;
}

void accelerometer_set_main_config(uint8_t config)
{
	wspi_write_byte(config, LIS302DL_CTRL_REG1_ADDR, GPIOE, GPIO3);
}

void accelerometer_set_filter_config(uint8_t config)
{
	wspi_write_byte(config, LIS302DL_CTRL_REG2_ADDR, GPIOE, GPIO3);
}

void accelerometer_interrupt_config(uint8_t config)
{
	wspi_write_byte(config, LIS302DL_CLICK_CFG_REG_ADDR, GPIOE, GPIO3);
}

void accelerometer_reset()
{
	uint8_t val = wspi_read_byte(LIS302DL_CTRL_REG2_ADDR, GPIOE, GPIO3);
	wspi_write_byte(val|LIS302DL_BOOT_REBOOTMEMORY, LIS302DL_CTRL_REG2_ADDR, GPIOE, GPIO3);
}

void accelerometer_filter_reset()
{
	wspi_read_byte(LIS302DL_HP_FILTER_RESET_REG_ADDR, GPIOE, GPIO3);
}

void accelerometer_read_raw_data(int8_t *values)
{
	uint8_t buffer[5];
	wspi_read_bytes(buffer, LIS302DL_OUT_X_ADDR, 5, GPIOE, GPIO3);
	values[0] = (int8_t)buffer[0];
	values[1] = (int8_t)buffer[2];
	values[2] = (int8_t)buffer[4];
}
