/** 
 ** @file   spi.c
 ** @date   2012-06-23
 ** @author Johannes Heimansberg (wejp.k.vu)
 **
 ** @brief  A simple Pong game for the STM32F4-Discovery board
 **         with external VGA DAC and accelerometer control.
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; version 2 of
 ** the License. See the file COPYING for details.
 **/
#include "spi.h"
#include "libopencm3/stm32/f4/gpio.h"
#include "libopencm3/stm32/f4/rcc.h"
#include "libopencm3/stm32/f4/spi.h"

#define READ_COMMAND 0x80 
#define MULTI_BYTE_COMMAND 0x40

static inline void lower_cs(uint32_t cs_port, uint16_t cs_pin)
{
	gpio_mode_setup(cs_port, GPIO_MODE_OUTPUT, GPIO_PUPD_PULLUP, cs_pin);
}

static inline void raise_cs(uint32_t cs_port, uint16_t cs_pin)
{
	gpio_mode_setup(cs_port, GPIO_MODE_INPUT, GPIO_PUPD_PULLUP, cs_pin);
}
#define TIMEOUT 0x1000

static uint8_t transfer_byte(uint8_t byte)
{
	uint32_t timer = TIMEOUT;
	while (!(SPI1_SR & SPI_SR_TXE)) if (timer-- == 0) return 0;

	SPI1_DR = byte;

	timer = TIMEOUT;
	while (!(SPI1_SR & SPI_SR_RXNE)) if (timer-- == 0) return 0;

	return SPI1_DR;
}

uint8_t wspi_read_byte(uint8_t address, uint32_t cs_port, uint16_t cs_pin)
{
	uint8_t byte;

	lower_cs(cs_port, cs_pin);
	transfer_byte(address|READ_COMMAND);
	byte = transfer_byte(0);
	raise_cs(cs_port, cs_pin);
	return byte;
}

void wspi_read_bytes(uint8_t *buffer, uint8_t address, int numbytes, uint32_t cs_port, uint16_t cs_pin)
{
	int i;

	if (numbytes > 1) address |= READ_COMMAND|MULTI_BYTE_COMMAND;
	else              address |= READ_COMMAND;

	lower_cs(cs_port, cs_pin);
	transfer_byte(address);
	for (i = 0; i < numbytes; i++) buffer[i] = transfer_byte(0);
	raise_cs(cs_port, cs_pin);
}

void wspi_write_byte(uint8_t byte, uint8_t address, uint32_t cs_port, uint16_t cs_pin)
{
	lower_cs(cs_port, cs_pin);
	transfer_byte(address);
	transfer_byte(byte);
	raise_cs(cs_port, cs_pin);
}

void wspi_write_bytes(uint8_t *bytes, uint8_t address, int numbytes, uint32_t cs_port, uint16_t cs_pin)
{
	int i;

	if (numbytes > 1) address |= MULTI_BYTE_COMMAND;
	lower_cs(cs_port, cs_pin);
	transfer_byte(address);
	for (i = 0; i < numbytes; i++) transfer_byte(bytes[i]);
	raise_cs(cs_port, cs_pin);
}
