/** 
 ** @file   vgapong.c
 ** @date   2012-06-23
 ** @author Johannes Heimansberg (wejp.k.vu)
 **
 ** @brief  A simple Pong game for the STM32F4-Discovery board
 **         with external VGA DAC and accelerometer control.
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
 ** as published by the Free Software Foundation; version 2 of
 ** the License. See the file COPYING for details.
 **/
#include <libopencm3/stm32/f4/rcc.h>
#include <libopencm3/stm32/f4/gpio.h>
#include "accelerometer.h"
#include "VGA.h"

inline void csleep(int t)
{
	int i;
	for (i = 0; i < t; i++)
		__asm__("nop");
}

void gpio_setup(void)
{
	rcc_peripheral_enable_clock(&RCC_AHB1ENR, RCC_AHB1ENR_IOPDEN);

	/* LEDs */
	gpio_set_output_options(GPIOD, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO12|GPIO13|GPIO14|GPIO15);
	gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO12);
	gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO13);
	gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO14);
	gpio_mode_setup(GPIOD, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO15);
	/* Button */
	rcc_peripheral_enable_clock(&RCC_AHB1ENR, RCC_AHB1ENR_IOPAEN);
	gpio_mode_setup(GPIOA, GPIO_MODE_INPUT, GPIO_PUPD_NONE, GPIO0);
}

uint8_t fb[320*200];

inline void set_pixel320(int16_t x, int16_t y, int8_t c)
{
	fb[x+320*y] = c;
}

#define PADDLE_LENGTH 32

inline void draw_paddle(int16_t x, int16_t y)
{
	int i;
	set_pixel320(x,   y, 0);
	set_pixel320(x+1, y, 0);
	for (i = y+1; i < y+PADDLE_LENGTH-1 && i < 200; i++) {
		set_pixel320(x,   i, 155);
		set_pixel320(x+1, i, 155);
	}
	set_pixel320(x,   y+PADDLE_LENGTH-1, 0);
	set_pixel320(x+1, y+PADDLE_LENGTH-1, 0);
}

inline void draw_ball(int16_t x, int16_t y)
{
	set_pixel320(x, y, 185);
}

inline void delete_ball(int16_t x, int16_t y)
{
	set_pixel320(x, y, 0);
}

int main(void)
{
	int player_paddle_y   = 200 / 2 - PADDLE_LENGTH / 2;
	int computer_paddle_y = 200 / 2 - PADDLE_LENGTH / 2;
	int c_dir = 1;
	int ball_x = 100, ball_y = 100, dir_x = 1, dir_y = 1;
	uint16_t button;

	InitializeSystem();
	gpio_setup();
	gpio_set(GPIOD, GPIO12); // green on
	InitializeVGAScreenMode400();

	accelerometer_init();
	csleep(1000000);
	accelerometer_set_main_config(0x47);
	csleep(1000000);

	/*int i, j;*/
	uint8_t *framebuffer1=fb;
	/*for (j = 0; j < 320*200-256; j+=256)
		for (i = 0; i < 256; i++)
			framebuffer1[j+i] = i;*/
	/* Rectangle */
	/*for (i = 100; i < 200; i++) {
		set_pixel320(i, 50,     155);
		set_pixel320(i, 150,    155);
		set_pixel320(100, i-50, 155);
		set_pixel320(200, i-50, 155);
	}*/
	SetVGAScreenMode320x200(framebuffer1);

	if (accelerometer_ping())
		gpio_clear(GPIOD, GPIO12); /* ping ok = green off */
	else
		gpio_set(GPIOD, GPIO12);   /* no valid ping reply = green on */

	draw_paddle(20, player_paddle_y);
	draw_paddle(300, computer_paddle_y);
	draw_ball(ball_x, ball_y);

	/* Wait for button press */
	do {
		button = GPIOA_IDR;
	} while (!(button & (1 << 0)));

	gpio_set(GPIOD, GPIO15); /* blue LED */
	/* Start game */
	while (1) {
		int8_t ad[3];
		WaitVBL();
		accelerometer_read_raw_data(ad);

		if (ad[1] > 40 && player_paddle_y < 200 - PADDLE_LENGTH - 1) player_paddle_y++;
		if (ad[1] < 20 && player_paddle_y > 0)                       player_paddle_y--;

		if (computer_paddle_y + PADDLE_LENGTH / 2 > ball_y) c_dir = -1; else c_dir = 1;
		computer_paddle_y += c_dir;
		if (computer_paddle_y < 0 || computer_paddle_y > 200 + PADDLE_LENGTH) computer_paddle_y -= c_dir;

		draw_paddle(20, player_paddle_y);
		draw_paddle(300, computer_paddle_y);
		delete_ball(ball_x, ball_y);
		ball_x += dir_x;
		ball_y += dir_y;
		if (ball_x < 20 && ball_y > player_paddle_y && ball_y < player_paddle_y + PADDLE_LENGTH) {
				ball_x -= dir_x;
				dir_x = -dir_x;
		}
		if (ball_x > 300 && ball_y > computer_paddle_y && ball_y < computer_paddle_y + PADDLE_LENGTH) {
			ball_x -= dir_x;
			dir_x = -dir_x;
		}
		if (ball_x > 318 || ball_x < 2) {
			ball_x -= dir_x;
			dir_x = -dir_x;
		}
		if (ball_y > 198 || ball_y < 2) {
			ball_y -= dir_y;
			dir_y = -dir_y;
		}
		draw_ball(ball_x, ball_y);
	}
	return 0;
}
